package de.intevation.flys.model;

import java.io.Serializable;
import java.util.List;

import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.OneToOne;
import javax.persistence.Table;

import org.hibernate.Session;
import org.hibernate.Query;
import org.hibernate.annotations.Type;

import com.vividsolutions.jts.geom.MultiLineString;

import de.intevation.flys.backend.SessionHolder;


/**
 * There is a modeling problem with the RiverAxis. The initial idea was, that a
 * river can have a riveraxis that consist of exact one geometry. Now, it has
 * turned out, that a single geometry is not enough for a riveraxis (arm of a
 * river, inflows, ...). As workaround, we now expect, that a river can just
 * have a single riveraxis.
 */
@Entity
@Table(name = "river_axes")
public class RiverAxis
implements   Serializable
{
    private Integer    id;
    private Integer    kind;
    private River      river;
    private MultiLineString geom;

    public static final int DEFAULT_KIND = 0;

    public static final int KIND_OFFICIAL = 1;
    public static final int KIND_OUTSOURCED = 2;

    public RiverAxis() {
    }


    @Id
    @Column(name = "id")
    public Integer getId() {
        return id;
    }


    public void setId(Integer id) {
        this.id = id;
    }


    @OneToOne
    @JoinColumn(name = "river_id")
    public River getRiver() {
        return river;
    }


    public void setRiver(River river) {
        this.river = river;
    }


    @Column(name = "kind")
    public Integer getKind() {
        return kind;
    }


    public void setKind(Integer kind) {
        this.kind = kind;
    }


    @Column(name = "geom")
    @Type(type = "org.hibernatespatial.GeometryUserType")
    public MultiLineString getGeom() {
        return geom;
    }


    public void setGeom(MultiLineString geom) {
        this.geom = geom;
    }


    public static List<RiverAxis> getRiverAxis(String river) {
        return getRiverAxis(river, DEFAULT_KIND);
    }

    public static List<RiverAxis> getRiverAxis(String river, int kind) {
        Session session = SessionHolder.HOLDER.get();

        Query query = session.createQuery(
            "from RiverAxis where river.name =:river AND kind =:kind");
        query.setParameter("river", river);
        query.setParameter("kind", kind);

        List<RiverAxis> list = query.list();

        return list.isEmpty() ? null : list;
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
